//
// sls.hh for sls
//
// Made by Seblu
// Login   <seblu@epita.fr>
//
// Started on  Mon Jun  4 14:32:04 2007 Seblu
// Last update Thu Jun  7 03:19:27 2007 Seblu
//

#ifndef SLS_HH
# define SLS_HH

# include <cassert>
# include <ios>
# include <iostream>
# include <fstream>
# include <sstream>
# include <map>

typedef std::string string;
typedef std::stringstream sstream;
typedef std::ostream ostream;
typedef std::ifstream ifstream;

enum {
  EXIT_OK = 0,
  EXIT_USAGE = 1,
  EXIT_FILE = 3,
  EXIT_UNKNOWN = 255
};

// -----------------------------------------------------------------------------
// ERROR CLASS
// -----------------------------------------------------------------------------

class Error
{
public:
  Error(int i);
  Error(string s = "", int i = EXIT_UNKNOWN);
  Error(const Error &e);
  void release() const;
  string message() const;
  int code() const;
  void code(int);
  Error &operator=(const Error &rhs);
  friend ostream &operator<<(ostream &o, const Error &e);
  friend Error &operator<<(Error &e, const string &s);
  friend Error &operator>>(Error &e, const string &s);
  friend Error &operator<<(Error &e, int val);
protected:
  sstream msg_;
  int val_;
};

// -----------------------------------------------------------------------------
// SLSERVER CLASS
// -----------------------------------------------------------------------------

class SLServer
{

  static const int backlog;

  struct AccessData
  {
    string client_pw;
    string server_pw;
    int privilege;
  };

  typedef std::map<string, AccessData *> AccessMap;

public:
  SLServer(int port);
  int run();

  void add_access(const char *login, const char *pw_server,
		 const char *pw_client, int priv);
  void del_access(const string &login);
  void print_access() const;
protected:
  int port_;
  AccessMap accessDB_;

  void connect();

};

// -----------------------------------------------------------------------------
// GETOPT CLASS
// -----------------------------------------------------------------------------
class GetOption
{
public:
  static void usage(const char *argv0);
  static const int min_argc;

  GetOption(int argv, char *argv[]);
  int port() const;
  bool verbose() const;
  const string& accessfile() const;

protected:
  int port_;
  string accessfile_;
  bool verbose_;
};

// -----------------------------------------------------------------------------
// PARSER CLASS
// -----------------------------------------------------------------------------
class Parser
{
public:
  Parser(const string filename, SLServer &server, bool verbose = false);
protected:
  string filename_;
  SLServer &server_;
  bool verbose_;
};

#endif
